#! /usr/bin/env python


from .device_dimmer import Device_Dimmer
from .device_insteon_base import Device_Insteon_Base

paddle_events = {'DON','DOF','DIM','BRT','DFON','DFOF'}

class Device_Insteon_Dimmer(Device_Dimmer,Device_Insteon_Base):

    def __init__(self, container, device_info):
        Device_Dimmer.__init__(self,container,device_info.name,device_info.address)
        Device_Insteon_Base.__init__(self, device_info)

        self.add_property('paddle_action')
        
        if device_info.property_value:        
            try:
                self.properties ['level'] = int(int(device_info.property_value)/255*100)
            except:
                pass

    def process_websocket_event(self,event):
            if event.control == 'ST':
                self.set_property('level',int(event.action)/255*100)
                #print ('device {}. changed status to {}'.format(self.name,event.action))

            elif event.control in paddle_events: #need to add other events
                self.set_property('paddle_action',event.control,True)
                #print ('device {}. changed local control {}'.format(self.name,event.action))

    def set_level(self,level):
        path = ('nodes/' + self.address + '/cmd/DON/' + str(int(level*255)))
        return self.send_request(path)

    def fast_on(self):
        path = ('nodes/' + self.address + '/cmd/DFON')
        return self.send_request(path)

    def fast_off(self):
        path = ('nodes/' + self.address + '/cmd/DFOF')
        return self.send_request(path)

    def brighten(self):
        path = ('nodes/' + self.address + '/cmd/BRT')
        return self.send_request(path)

    def dim(self):
        path = ('nodes/' + self.address + '/cmd/DIM')
        return self.send_request(path)



#! /usr/bin/env python



from .device_base import Device_Base


class Device_Switch(Device_Base):

    def __init__(self, container,name,address):
        Device_Base.__init__(self,container,'switch',name,address)

        self.add_property('onoff','off')

    def turn_on(self):
        pass

    def turn_off(self):
        pass






#! /usr/bin/env python

import xml.etree.ElementTree as ET

import traceback

# process a node and assemble the info to build a device


''' zwave node....
<nodeInfo>
<node flag="0">
<address>ZW002_143</address>
<name>ZW 002 Energy Meter</name>
<family>4</family>
<type>4.33.1.0</type>
<enabled>true</enabled>
<deviceClass>0</deviceClass>
<wattage>0</wattage>
<dcPeriod>0</dcPeriod>
<startDelay>0</startDelay>
<endDelay>0</endDelay>
<pnode>ZW002_1</pnode>
<sgid>143</sgid>
<devtype>
<gen>4.33.1</gen>
<mfg>134.2.9</mfg>
<cat>143</cat>
</devtype>
<ELK_ID>I16</ELK_ID>
<property id="ST" value="91289" formatted="912.89 Watts" uom="73" prec="2"/>
</node>
<properties>
<property id="ST" value="91289" formatted="912.89 Watts" uom="73" prec="2"/>
<property id="TPW" value="31765" formatted="317.65 kWh" uom="33" prec="2"/>
</properties>
</nodeInfo>
'''

class Device_Info(object):

    def __init__(self,node):

        self.node = node

        self.valid = False

        try:
            self.family = '1' # default to insteon
            if node.find('family'): # insteon does not set node family
                self.family = node.find('family').text

            type_ = node.find('type')
            self.type = type_.text
            types = type_.text.split('.')
            self.category = types [0]    
            self.sub_category = types [1]    
            self.version = types [2]    
            self.reserved = types [3] 

            address = node.find('address')
            self.address = address.text
            self.address_parts = address.text.split(' ')

            name = node.find('name')
            self.name = name.text

            flag = node.get ('flag')
            self.flag = int(flag) 
            self.node_def_id = node.get('nodeDefId')

            #container_node = node.find('container')
            #self.container_node_address = container_node.text  
        
            property_node = node.find('property')
            if 'value' in property_node.attrib:
                self.property_value = property_node.attrib['value']
            else:
                self.property_value = None

            self.valid = True

            #print (self)
        
        except Exception as ex:
                traceback.print_exc()       

    def __repr__(self):
        return 'Device: Name {} Address {}, Family {}, Type {}, Def ID {}'.format(self.name,self.address,self.family,self.type,self.node_def_id)
    


#! /usr/bin/env python


from .device_base import Device_Base


class Device_Dimmer(Device_Base):

    def __init__(self, container, name, address):
        Device_Base.__init__(self,container,'dimmer', name, address)

        self.add_property('level',0) #in percent

    def set_level(self,level):
        pass




#! /usr/bin/env python

''' 

Base Program

<eventInfo>
    <id>1C</id>
    <on />
    <nr />
    <r>190520 08:28:16</r>
    <f>190520 08:28:16</f>
    <s>31</s>
</eventInfo>

'''
from .. item_base import Item_Base

class Program_Base(Item_Base):

    def __init__(self, container, program_info):
        Item_Base.__init__(self,container,program_info.name)

        self.id = program_info.id

        self.add_property('state', 'idle') #program run state
        self.add_property('last_run_time', program_info.last_run_time)
        self.add_property('last_finish_time', program_info.last_finish_time)

    def __str__(self):
        return ("Program: {}; id {}; run {}, finish {}".format(self.name, self.id, self.properties['last_run_time'], self.properties['last_finish_time']))

    def process_websocket_event(self,state_code,last_run_time,last_finish_time):
        state = 'unknown'
        if state_code & 1 == 1:
            state = 'idle'
        elif state_code & 2 == 2:
            state = 'then'
        elif state_code & 3 == 3:
            state = 'else'

        self.set_property('state',state)
        self.set_property('last_run_time',last_run_time)
        self.set_property('last_finish_time',last_finish_time)

    def command(self,command):
        path = 'programs/' + self.id + '/' + command
        return self.send_request(path)        

    def run(self):
        self.command('run')

    def stop(self):
        self.command('stop')

    def run_then(self):
        self.command('runThen')

    def run_else(self):
        self.command('runElse')

    def enable(self):
        self.command('enable')

    def disable(self):
        self.command('disable')
        
    def get_identifier(self):
        return self.id        
        


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(name='ISY994v5',
      version='0.1.5',
      description='ISY99 Controller Rest and Websocket client v5 firmware',
      author='Michael Cumming',
      author_email='mike@4831.com',
      long_description=long_description,
      long_description_content_type="text/markdown",      
      url='https://github.com/mjcumming/ISY994v5',
      keywords = ['INSTEON','ISY994'],  
      packages=setuptools.find_packages(exclude=("test",)),
      classifiers=[
            "Programming Language :: Python :: 3",
            "License :: OSI Approved :: MIT License",
            "Operating System :: OS Independent",
      ],      
    install_requires=[
        'requests',
        'websocket_client',
    ]      
)



#! /usr/bin/env python
import xml.etree.ElementTree as ET

from .. item_container import Item_Container
from .variable_info import Variable_Info
from .variable_name import Variable_Name
from .variable_integer import Variable_Integer
from .variable_state import Variable_State

import logging
logger = logging.getLogger(__name__)

variable_classes = {
    '1' : Variable_Integer,
    '2' : Variable_State,
} 

class Variable_Container (Item_Container):

    def __init__(self, controller):
        Item_Container.__init__(self,controller,'Variable')

    def start(self):
        success = True

        success_list,variable_list_response = self.controller.send_request('vars/get/1') # get integer vars
        success_name,variable_name_response = self.controller.send_request('vars/definitions/1') # get integer vars

        if success_list and success_name and variable_list_response.status_code == 200 and variable_name_response.status_code == 200:
            list_root = ET.fromstring (variable_list_response.content)        
            name_root = ET.fromstring (variable_name_response.content)        
            self.process_variable_nodes (list_root,name_root)       
        else:
            success = False

        success_list,variable_list_response = self.controller.send_request('vars/get/2') # get state vars
        success_name,variable_name_response = self.controller.send_request('vars/definitions/2') # get state vars

        if success_list and success_name and variable_list_response.status_code == 200 and variable_name_response.status_code == 200:
            list_root = ET.fromstring (variable_list_response.content)        
            name_root = ET.fromstring (variable_name_response.content)        
            self.process_variable_nodes (list_root,name_root)       
        else:
            success = False

        if success:
            self.items_retrieved = True
            return True
        else:
            return False

    def process_variable_nodes(self,list_root,name_root):
        for node in list_root.iter('var'):
            variable_info = Variable_Info(node)

            if variable_info.valid: # make sure we have the info we need
                variable_class = variable_classes [variable_info.type]
                variable = variable_class(self,variable_info)

                #find name
                for node in name_root.iter('e'):
                    variable_name = Variable_Name (node)

                    if variable_name.valid and variable_name.id == variable.id:
                        variable.name = variable_name.name

                self.add(variable,variable.get_index())
         
    def websocket_event(self,event):
        var = event.event_info_node.find('var')
        variable_id = var.attrib ['id']
        variable_type = var.attrib ['type']

        index = variable_type+':'+variable_id

        variable = self.get(index)
        if variable is not None:
            variable.process_websocket_event(event)
        else:
            logger.warning('Unable able to find variable type {} id {}'.format(variable_id,variable_type))

